/**
 * Copyright (c) 2015 - 2020, Nordic Semiconductor ASA
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 *
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
/** @file
 * @defgroup pwm_example_main main.c
 * @{
 * @ingroup pwm_example
 *
 * @brief  PWM Example Application main file.
 *
 * This file contains the source code for a sample application using PWM.
 *
 *
 */

#include <stdbool.h>
#include <stdint.h>
#include "nrf.h"
#include "app_error.h"
#include "bsp.h"
#include "nrf_delay.h"

#include "nrfx_timer.h"
#include "nrfx_gpiote.h"
#include "nrfx_ppi.h"

#define TOGGLE_DELAY_US        200000             //starting out at 2s for easy visualization

#define PIN_IN BSP_BUTTON_0
#define PIN_OUT BSP_LED_0

const nrfx_timer_t m_gpiote_timer = NRFX_TIMER_INSTANCE(0);

static nrf_ppi_channel_t    m_timer_cc_pin_out_ppi_channel;

void input_pin_event_handler(nrfx_gpiote_pin_t pin, nrf_gpiote_polarity_t action)
{
    nrfx_timer_enable(&m_gpiote_timer);
}

void delay_timer_handler(nrf_timer_event_t event_type, void* p_context)
{
    UNUSED_PARAMETER(p_context);
}

void timers_init(void)
{
    uint32_t err_code;
    uint32_t toggle_delay_ticks;
    
    nrfx_timer_config_t timer_cfg = NRFX_TIMER_DEFAULT_CONFIG;
    err_code = nrfx_timer_init(&m_gpiote_timer, &timer_cfg, delay_timer_handler);
    APP_ERROR_CHECK(err_code);

    toggle_delay_ticks = nrfx_timer_us_to_ticks(&m_gpiote_timer, TOGGLE_DELAY_US);

    nrfx_timer_extended_compare(&m_gpiote_timer, 
                                  NRF_TIMER_CC_CHANNEL0, 
                                  toggle_delay_ticks,
                                  NRF_TIMER_SHORT_COMPARE0_CLEAR_MASK,
                                  false);

     //set up toggling of PIN_OUT when timer expires
     uint32_t timer_compare_event_addr = nrfx_timer_compare_event_address_get(&m_gpiote_timer, NRF_TIMER_CC_CHANNEL0);
     
     uint32_t gpiote_out_toggle_task_addr = nrfx_gpiote_out_task_addr_get(PIN_OUT);

     err_code = nrfx_ppi_channel_alloc(&m_timer_cc_pin_out_ppi_channel);
     APP_ERROR_CHECK(err_code);

     err_code = nrfx_ppi_channel_assign(m_timer_cc_pin_out_ppi_channel,
                                        timer_compare_event_addr,
                                        gpiote_out_toggle_task_addr);
     APP_ERROR_CHECK(err_code);    
}

static void gpio_init(void)
{
    uint32_t err_code;

    err_code = nrfx_gpiote_init();
    APP_ERROR_CHECK(err_code);

    nrfx_gpiote_in_config_t in_config = NRFX_GPIOTE_CONFIG_IN_SENSE_TOGGLE(false);
    in_config.pull = NRF_GPIO_PIN_PULLUP;
    
    err_code = nrfx_gpiote_in_init(PIN_IN, &in_config, input_pin_event_handler);
    APP_ERROR_CHECK(err_code);

    nrfx_gpiote_in_event_enable(PIN_IN, false);

    nrfx_gpiote_out_config_t out_config = NRFX_GPIOTE_CONFIG_OUT_TASK_TOGGLE(false);
    
    err_code = nrfx_gpiote_out_init(PIN_OUT, &out_config);
    APP_ERROR_CHECK(err_code);

    nrfx_gpiote_out_task_enable(PIN_OUT);

}

void ppi_channels_enable(void)
{
    uint32_t err_code;

    err_code = nrfx_ppi_channel_enable(m_timer_cc_pin_out_ppi_channel);
    APP_ERROR_CHECK(err_code);
}

int main(void)
{
    uint32_t value;
    uint32_t err_code;
    
    gpio_init();
    timers_init();

    ppi_channels_enable();

    while (true)
    {

    }

}


/** @} */
